package mutator

import (
	"crypto/md5"
	"encoding/binary"
	"fmt"
	"io"
	"math/rand"

	"bitbucket.org/nimblebet/raystuff/internal/enum"
	"bitbucket.org/nimblebet/raystuff/internal/game"
	"bitbucket.org/nimblebet/raystuff/internal/generator"
	rl "github.com/gen2brain/raylib-go/raylib"
)

func LoadFirstWorld(c game.Config, s game.State) game.State {
	h := md5.New()
	io.WriteString(h, s.Username)
	seed := binary.BigEndian.Uint64(h.Sum(nil))
	rand.Seed(int64(seed))

	randBottlesTextures := []int{}
	var inSlice = func(slice []int, val int) bool {
		for _, item := range slice {
			if item == val {
				return true
			}
		}
		return false
	}

	for {
		n := rand.Intn(len(c.Textures.BotlesRand))
		if !inSlice(randBottlesTextures, n) {
			randBottlesTextures = append(randBottlesTextures, n)
		}

		if len(randBottlesTextures) == len(c.Textures.BotlesRand) {
			break
		}
	}
	fmt.Println(generator.Map(c.LabMapSize))
	s.CurrentLevel = 1
	s.World = game.World{
		Map: generator.Map(c.LabMapSize),

		Player: game.Entity{
			WorkTube: game.WorkTube{
				Size: 100,
			},
			Substances: map[enum.SubstanceType]game.Substance{
				enum.FamilyType: game.Substance{
					StockLevel: 100,
					Texture:    c.Textures.BotlesRand[randBottlesTextures[0]],
					Position:   rl.NewVector2(290, 350),
				},
				enum.HeightType: game.Substance{
					StockLevel: 100,
					Texture:    c.Textures.BotlesRand[randBottlesTextures[1]],
					Position:   rl.NewVector2(370, 350),
				},
				enum.CalorificValueType: game.Substance{
					StockLevel: 100,
					Texture:    c.Textures.BotlesRand[randBottlesTextures[2]],
					Position:   rl.NewVector2(450, 350),
				},
			},
			Position: rl.RectangleInt32{
				X: 15,
				Y: 11,
			},
		},
		Lab: game.Lab{
			SelectedSubstance: enum.FamilyType,
		},
		Garden: game.Garden{
			CurrentCellIndex: 1,
			ColumnsCount:     c.Garden.ColumnsCount,
			RowsCount:        c.Garden.RowsCount,
			Cells: []game.GardenCell{
				game.GardenCell{},
				game.GardenCell{},
				game.GardenCell{},
				game.GardenCell{},
				game.GardenCell{},
				game.GardenCell{},
				game.GardenCell{},
				game.GardenCell{},
			},
		},
		PlantToGrow: c.Plants[0],
	}

	return s
}
