package main

import (
	"fmt"

	"bitbucket.org/nimblebet/raystuff/internal/initializer"

	"bitbucket.org/nimblebet/raystuff/internal/enum"
	"bitbucket.org/nimblebet/raystuff/internal/game"
	"bitbucket.org/nimblebet/raystuff/internal/layer"
	rl "github.com/gen2brain/raylib-go/raylib"
)

func main() {
	c, err := initGameConfig()
	if err != nil {
		fmt.Println("failed to init the config:", err)
		return
	}

	processMenuEvents := initializer.MenuMux()
	processLabEvents := initializer.LabMux()
	processGlobalEvents := initializer.GlobalMux()
	processGardenEvents := initializer.GardenMux()
	processLevelPassedEvents := initializer.LevelPassedMux()
	processBriefingEvents := initializer.BriefingMux()
	processWinEvents := initializer.WinMux()
	processLoseEvents := initializer.LoseMux()
	processTaskWindowEvents := initializer.TaskWindowMux()

	s := game.State{
		CurrentScreen: enum.Briefing,
	}

	rl.InitAudioDevice()

	s.DoomsdaySong = rl.LoadMusicStream("./assets/doomsday-hackathon-song.ogg")
	s.FinalSong = rl.LoadMusicStream("./assets/final-splash.ogg")
	s.WinSplash = rl.LoadMusicStream("./assets/win-splash.ogg")
	s.LoseSplash = rl.LoadMusicStream("./assets/lose-splash.ogg")

	s.CurrentSong = s.DoomsdaySong

	rl.PlayMusicStream(s.DoomsdaySong)

	for !rl.WindowShouldClose() {
		rl.BeginDrawing()
		rl.ClearBackground(rl.Black)

		switch s.CurrentScreen {
		case enum.Splash:
		case enum.Briefing:
			layer.Briefing(c, s)

			s = processBriefingEvents(c, s)

		case enum.Win:
			layer.Win(c, s)

			s = processWinEvents(c, s)

		case enum.Lose:
			layer.Lose(c, s)

			s = processLoseEvents(c, s)

		case enum.LevelPassed:
			layer.LevelPassed(c, s)

			s = processLevelPassedEvents(c, s)

		case enum.Menu:
			layer.MenuUI(c, s)

			s = processMenuEvents(c, s)

		case enum.Lab:
			layer.Lab(c, s)

			s = processLabEvents(c, s)

		case enum.Garden:
			layer.Garden(c, s)

			s = processGardenEvents(c, s)

		case enum.TaskDescription:
			layer.TaskWindow(c, s)

			s = processTaskWindowEvents(c, s)
		}

		layer.BackgroundMusic(c, s)
		layer.Message(c, s)

		s = processGlobalEvents(c, s)

		rl.EndDrawing()
	}

	unloadTextures(c)
	rl.CloseAudioDevice()

	rl.CloseWindow()
}
